from .analytics import (
    ClientsBarChartViewSet,
    CountryBarChartViewSet,
    MailClickBarChartViewSet,
    MailStatusBarChartViewSet,
    RegionBarChartViewSet,
)
from .mailing_lists import (
    EmailContactMailingListModelViewSet,
    MailingListEmailContactThroughModelModelViewSet,
    MailingListEntryModelViewSet,
    MailingListModelViewSet,
    MailingListRepresentationViewSet,
    MailingListSubscriberChangeRequestModelViewSet,
    MailingListSubscriberRequestEntryModelViewSet,
    MailingListSubscriberRequestMailingListModelViewSet,
    ManageMailingListSubscriptions,
    UnsubscribeView,
)
from .mails import (
    MailEventModelViewSet,
    MailEventMailModelViewSet,
    MailEventMassMailMailModelViewSet,
    MailMailingListChartViewSet,
    MailModelViewSet,
    MailRepresentationViewSet,
    MailStatusMassMailModelViewSet,
    MailTemplateModelViewSet,
    MailTemplateRepresentationViewSet,
    MassMailModelViewSet,
    MassMailRepresentationViewSet,
)



from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from .models import (
    Mail,
    MailEvent,
    MailingList,
    MailingListEmailContactThroughModel,
    MailingListSubscriberChangeRequest,
    MailTemplate,
    MassMail,
)


class MailingListEmailContactThroughInlineAdmin(admin.TabularInline):
    model = MailingListEmailContactThroughModel
    fk_name = "mailing_list"
    autocomplete_fields = ["email_contact"]


@admin.register(MailingList)
class MailingListAdmin(admin.ModelAdmin):
    autocomplete_fields = ["email_contacts"]
    search_fields = ["title"]
    inlines = [
        MailingListEmailContactThroughInlineAdmin,
    ]


@admin.register(MassMail)
class MassMailAdmin(admin.ModelAdmin):
    autocomplete_fields = ["creator"]

    def send_test_mail(self, request, queryset):
        for mass_mail in queryset:
            mass_mail.send_test_mail(request.user)

    actions = [send_test_mail]


@admin.register(MailingListSubscriberChangeRequest)
class MailingListSubscriberChangeRequestAdmin(admin.ModelAdmin):
    list_display = ("email_contact", "mailing_list", "status")
    autocomplete_fields = ["email_contact", "requester"]


admin.site.register(MailEvent)


class MailEventInline(admin.TabularInline):
    model = MailEvent
    fields = ("timestamp", "event_type", "reject_reason", "recipient", "user_agent", "tags", "description")
    readonly_fields = ("timestamp", "event_type", "reject_reason", "recipient", "user_agent", "tags", "description")
    extra = 0
    can_delete = False


@admin.register(Mail)
class MailAdmin(admin.ModelAdmin):
    def send_mails(self, request, queryset):
        for mail in queryset:
            mail.resend()

    send_mails.short_description = _("Send Emails")
    actions = [send_mails]
    search_fields = ["mass_mail__subject", "from_email", "to_email__address", "subject", "message_ids"]
    autocomplete_fields = ["to_email", "cc_email", "bcc_email"]

    inlines = [MailEventInline]


@admin.register(MailTemplate)
class MailTemplateAdmin(admin.ModelAdmin):
    pass



from django.dispatch import receiver
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers
from rest_framework.reverse import reverse
from wbcore import serializers as wb_serializers
from wbcore.contrib.directory.serializers import (
    CompanyModelSerializer,
    EmailContactRepresentationSerializer,
    EntryModelSerializer,
    PersonModelSerializer,
    PersonRepresentationSerializer,
)
from wbcore.signals.serializers import add_instance_additional_resource

from wbmailing import models
from wbmailing.models import MailingListEmailContactThroughModel


class MailingListRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbmailing:mailinglist-detail")

    class Meta:
        model = models.MailingList
        fields = ("id", "title", "_detail")


class MailingListEntryRepresentationSerializer(MailingListRepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbmailing:mailinglist-detail")

    def get_filter_params(self, request):
        entry_id = request.parser_context["view"].kwargs.get("entry_id", None)
        return {"negative_entry": entry_id}

    class Meta:
        model = models.MailingList
        fields = ("id", "title", "_detail")


class MailingListListSerializer(wb_serializers.ModelSerializer):
    nb_subscribers = wb_serializers.IntegerField(read_only=True, label=_("Number of Subscribers"), default=0)

    class Meta:
        model = models.MailingList
        fields = (
            "id",
            "title",
            "is_public",
            "nb_subscribers",
        )


class MailingListModelSerializer(wb_serializers.ModelSerializer):
    nb_subscribers = wb_serializers.IntegerField(read_only=True, label=_("Number of Subscribers"), default=0)

    @wb_serializers.register_resource()
    def email_contacts(self, instance, request, user):
        # Do some something (checks, etc.)
        return {
            "mailevent_chart": reverse(
                "wbmailing:mailing_list-maileventchart-list", args=[instance.id], request=request
            ),
            "email_contacts": reverse(
                "wbmailing:mailing_list-email_contacts-list", args=[instance.id], request=request
            ),
        }

    class Meta:
        model = models.MailingList
        fields = ("id", "title", "is_public", "nb_subscribers", "_additional_resources")


class MailingListEmailContactThroughModelModelSerializer(wb_serializers.ModelSerializer):
    _email_contact = EmailContactRepresentationSerializer(source="email_contact")
    _mailing_list = MailingListRepresentationSerializer(source="mailing_list")

    in_charge = wb_serializers.CharField(read_only=True, label=_("Relationship Managers"))
    expiration_date = wb_serializers.DateField(read_only=True, label=_("Expiration Date"))
    is_public = wb_serializers.BooleanField(read_only=True, label=_("Public Mailing list"))
    is_pending_request_change = wb_serializers.BooleanField(
        read_only=True,
        default=False,
        label=_("Pending change request exists"),
        help_text=_("If true, a pending change request exists for this email and mailing list"),
    )

    @wb_serializers.register_resource()
    def delete_from_mailinglist(self, instance, request, user):
        res = {
            "requests": f'{reverse("wbmailing:mailinglistsubscriberchangerequest-list", args=[], request=request)}?email_contact={instance.email_contact.id}&mailing_list={instance.mailing_list.id}'
        }
        if getattr(instance, "expiration_date", None):
            res["remove_expiration_date"] = reverse(
                "wbmailing:mailinglistemailcontact-removeexpirationdate",
                args=[instance.id],
                request=request,
            )
        if instance.status != MailingListEmailContactThroughModel.Status.SUBSCRIBED:
            res["delete_from_mailinglist"] = reverse(
                "wbmailing:mailinglistemailcontact-delete",
                args=[instance.id],
                request=request,
            )
        else:
            res["unsubscribe"] = reverse(
                "wbmailing:mailinglistemailcontact-unsubscribe",
                args=[instance.id],
                request=request,
            )
        return res

    class Meta:
        model = MailingListEmailContactThroughModel
        fields = (
            "id",
            "status",
            "email_contact",
            "mailing_list",
            "_email_contact",
            "_mailing_list",
            "in_charge",
            "expiration_date",
            "is_public",
            "is_pending_request_change",
            "_additional_resources",
        )


class MailingListSubscriberChangeRequestModelSerializer(wb_serializers.ModelSerializer):
    _email_contact = EmailContactRepresentationSerializer(many=False, source="email_contact")
    _mailing_list = MailingListRepresentationSerializer(many=False, source="mailing_list")
    _requester = PersonRepresentationSerializer(source="requester", many=False)
    _approver = PersonRepresentationSerializer(source="approver", many=False)

    approver = wb_serializers.PrimaryKeyRelatedField(read_only=True, label=_("Approver"))
    requester = wb_serializers.PrimaryKeyRelatedField(read_only=True, label=_("Requester"))
    created = wb_serializers.DateTimeField(read_only=True, label=_("Created"))
    updated = wb_serializers.DateTimeField(read_only=True, label=_("Updated"))
    entry_repr = wb_serializers.CharField(read_only=True)
    type = wb_serializers.ChoiceField(choices=models.MailingListSubscriberChangeRequest.Type.choices, required=False)

    def validate(self, attrs):
        instance_id = self.instance.id if self.instance else None
        email_contact = attrs.get("email_contact", self.instance.email_contact if self.instance else None)
        mailing_list = attrs.get("mailing_list", self.instance.mailing_list if self.instance else None)
        if email_contact and mailing_list:
            if (
                models.MailingListSubscriberChangeRequest.objects.exclude(id=instance_id)
                .filter(
                    email_contact=email_contact,
                    mailing_list=mailing_list,
                    status=models.MailingListSubscriberChangeRequest.Status.PENDING.name,
                )
                .count()
                > 0
            ):
                raise serializers.ValidationError(
                    {
                        "non_field_errors": gettext("There is already a pending request to subscribe {} to {}").format(
                            email_contact.address, mailing_list.title
                        )
                    }
                )
        if not email_contact:
            raise serializers.ValidationError(
                {"email_contact": gettext("Email contact is missing for this request to be valid")}
            )
        return super().validate(attrs)

    def create(self, validated_data):
        if request := self.context.get("request"):
            validated_data["requester"] = request.user.profile
        return super().create(validated_data)

    class Meta:
        model = models.MailingListSubscriberChangeRequest
        fields = (
            "id",
            "status",
            "email_contact",
            "_email_contact",
            "email_contact",
            "mailing_list",
            "_mailing_list",
            "expiration_date",
            "type",
            "created",
            "updated",
            "requester",
            "_requester",
            "approver",
            "_approver",
            "reason",
            "entry_repr",
            "_additional_resources",
        )


@receiver(add_instance_additional_resource, sender=CompanyModelSerializer)
@receiver(add_instance_additional_resource, sender=PersonModelSerializer)
@receiver(add_instance_additional_resource, sender=EntryModelSerializer)
def crm_adding_additional_resource(sender, serializer, instance, request, user, **kwargs):
    return {
        "mailinglist": reverse("wbmailing:entry-mailinglist-list", args=[instance.id], request=request),
        "mailinglistsubscriptionrequests": reverse(
            "wbmailing:entry-mailinglistsubscriberchangerequest-list",
            args=[instance.id],
            request=request,
        ),
    }



from django.apps import apps
from django.db.models import Exists, OuterRef
from django.utils.translation import gettext_lazy as _
from wbcore import filters as wb_filters
from wbcore.contrib.directory.models import EmailContact, Entry

from wbmailing import models


class MailingListFilterSet(wb_filters.FilterSet):
    not_factsheet_mailinglist = wb_filters.BooleanFilter(
        label=_("No Factsheet Mailing Lists"), method="boolean_not_factsheet_mailinglist", default=True
    )
    negative_entry = wb_filters.ModelChoiceFilter(
        label=_("Unsubscribed mailing lists for user"),
        queryset=Entry.objects.all(),
        endpoint=Entry.get_representation_endpoint(),
        value_key=Entry.get_representation_value_key(),
        label_key=Entry.get_representation_label_key(),
        method="get_notsubscribed_mailing_list_for_entry",
    )

    def get_notsubscribed_mailing_list_for_entry(self, queryset, name, value):
        if value:
            already_subscribed_subquery = models.MailingListEmailContactThroughModel.objects.filter(
                mailing_list=OuterRef("pk"),
                email_contact__in=value.emails.all(),
                status__in=[models.MailingListEmailContactThroughModel.Status.SUBSCRIBED],
            )
            return queryset.annotate(already_subscribed_subquery=Exists(already_subscribed_subquery)).filter(
                already_subscribed_subquery=False
            )

        return queryset

    def boolean_not_factsheet_mailinglist(self, queryset, name, value):
        if apps.is_installed("wbreport"):
            if value is False:
                return queryset.filter(reports__isnull=False).distinct()
            elif value is True:
                return queryset.filter(reports__isnull=True).distinct()
        return queryset

    class Meta:
        model = models.MailingList
        fields = {"email_contacts": ["exact"], "is_public": ["exact"]}


class MailingListSubscriberChangeRequestFilterSet(wb_filters.FilterSet):
    class Meta:
        model = models.MailingListSubscriberChangeRequest
        fields = {
            "email_contact": ["exact"],
            "mailing_list": ["exact"],
            "requester": ["exact"],
            "created": ["gte", "exact", "lte"],
        }


class MailStatusMassMailFilterSet(wb_filters.FilterSet):
    status = wb_filters.ChoiceFilter(label=_("Status"), choices=models.MailEvent.EventType.choices)

    class Meta:
        model = EmailContact
        fields = {"entry": ["exact"]}


class MailingListEmailContactThroughModelModelFilterSet(wb_filters.FilterSet):
    expiration_date = wb_filters.DateTimeRangeFilter(
        label=_("Expiration Date"),
        method=wb_filters.DateRangeFilter.base_date_range_filter_method,
    )
    is_pending_request_change = wb_filters.BooleanFilter(
        label=_("Pending Change"), lookup_expr="exact", field_name="is_pending_request_change"
    )

    class Meta:
        model = models.MailingListEmailContactThroughModel
        fields = {"status": ["exact"]}


class EmailContactMailingListFilterSet(MailingListEmailContactThroughModelModelFilterSet):
    class Meta:
        model = models.MailingListEmailContactThroughModel
        fields = {"email_contact": ["exact"], "status": ["exact"]}
